/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSAppleScript;

import dstep.foundation.NSAppleEventDescriptor;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSAppleScript_bindings;

private 
{
	NSString NSAppleScriptErrorMessage_;
	NSString NSAppleScriptErrorNumber_;
	NSString NSAppleScriptErrorAppName_;
	NSString NSAppleScriptErrorBriefMessage_;
	NSString NSAppleScriptErrorRange_;
}

NSString NSAppleScriptErrorMessage ()
{
	if (NSAppleScriptErrorMessage_)
		return NSAppleScriptErrorMessage_;
		
	return NSAppleScriptErrorMessage_ = new NSString(bindings.NSAppleScriptErrorMessage);
}

NSString NSAppleScriptErrorNumber ()
{
	if (NSAppleScriptErrorNumber_)
		return NSAppleScriptErrorNumber_;
		
	return NSAppleScriptErrorNumber_ = new NSString(bindings.NSAppleScriptErrorNumber);
}

NSString NSAppleScriptErrorAppName ()
{
	if (NSAppleScriptErrorAppName_)
		return NSAppleScriptErrorAppName_;
		
	return NSAppleScriptErrorAppName_ = new NSString(bindings.NSAppleScriptErrorAppName);
}

NSString NSAppleScriptErrorBriefMessage ()
{
	if (NSAppleScriptErrorBriefMessage_)
		return NSAppleScriptErrorBriefMessage_;
		
	return NSAppleScriptErrorBriefMessage_ = new NSString(bindings.NSAppleScriptErrorBriefMessage);
}

NSString NSAppleScriptErrorRange ()
{
	if (NSAppleScriptErrorRange_)
		return NSAppleScriptErrorRange_;
		
	return NSAppleScriptErrorRange_ = new NSString(bindings.NSAppleScriptErrorRange);
}

class NSAppleScript : NSObject, INSCopying
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithContentsOfURL (NSURL url, ref NSDictionary errorInfo)
	{
		id error;
		
		if (errorInfo)
			error = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "initWithContentsOfURL:error:", NSURL, id*)(url, &error);
				
		if (error)
			errorInfo = new NSDictionary(error);
		
		return result;
	}

	this (NSURL url, ref NSDictionary errorInfo)
	{
		typeof(this).alloc.initWithContentsOfURL(url, errorInfo);
	}

	Object initWithSource (NSString source)
	{
		return invokeObjcSelf!(Object, "initWithSource:", NSString)(source);
	}

	this (NSString source)
	{
		typeof(this).alloc.initWithSource(source);
	}

	NSString source ()
	{
		return invokeObjcSelf!(NSString, "source");
	}

	bool isCompiled ()
	{
		return invokeObjcSelf!(bool, "isCompiled");
	}

	bool compileAndReturnError (ref NSDictionary errorInfo)
	{
		id error;
		
		if (errorInfo)
			error = new objc_object;
		
		bool result =  invokeObjcSelf!(bool, "compileAndReturnError:", id*)(&error);
		
		if (error)
			errorInfo = new NSDictionary(error);
		
		return result;
	}

	NSAppleEventDescriptor executeAndReturnError (ref NSDictionary errorInfo)
	{
		id error;
		
		if (errorInfo)
			error = new objc_object;
		
		NSAppleEventDescriptor result = invokeObjcSelf!(NSAppleEventDescriptor, "executeAndReturnError:", id*)(&error);
		
		if (error)
			errorInfo = new NSDictionary(error);
		
		return result;
	}

	NSAppleEventDescriptor executeAppleEvent (NSAppleEventDescriptor event, ref NSDictionary errorInfo)
	{
		id error;
		
		if (errorInfo)
			error = new objc_object;
		
		NSAppleEventDescriptor result = invokeObjcSelf!(NSAppleEventDescriptor, "executeAppleEvent:error:", NSAppleEventDescriptor, id*)(event, &error);
		
		if (error)
			errorInfo = new NSDictionary(error);
		
		return result;
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}
}

